/* -*- mode: c++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/*
 Copyright (C) 2011 Klaus Spanderen

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.

 Part of the example are taken from the QuantLib, 

 compile:
export LD_LIBRARY_PATH=/usr/lib/R/lib:/usr/lib/R/library/RInside/lib:/usr/lib/R/library/Rcpp/lib:/usr/lib/R/library/RInside/lib:/usr/lib/R/library/Rcpp/lib


g++ -I/usr/lib/R/include -I/usr/lib/R/library/RInside/include -I/usr/lib/R/library/Rcpp/include -I/home/spanderen/workspace/QuantLib -I/usr/local/include/boost-1_38 -O3 -o vpp vpp.cpp -L/home/spanderen/workspace/QuantLib/ql/.libs -lQuantLib  -L/usr/lib/R/library/RInside/lib -lRInside -L/usr/lib/R/library/Rcpp/lib -lRcpp -L/usr/lib/R/lib -lR 
*/


#include <ql/math/functional.hpp>
#include <ql/time/daycounters/actualactual.hpp>
#include <ql/math/randomnumbers/rngtraits.hpp>
#include <ql/experimental/processes/extouwithjumpsprocess.hpp>
#include <ql/experimental/processes/extendedornsteinuhlenbeckprocess.hpp>

#include <RInside.h>  // for the embedded R via RInside

#include <fstream>

using namespace QuantLib;

namespace {

    const Real beta = 200;
    const Real eta  = 1.0/0.4;
    const Real lambda = 4.0;
    const Real alpha = 7.0;
    const Real volatility_x = 1.4;
    const Real kappa = 4.45;
    const Real volatility_u = std::sqrt(1.3);
    const Real rho = 0.7;
    const Rate irRate = 0.00;
}

int main(int argc, char *argv[]) {
    const DayCounter dc = ActualActual();
    const Date settlementDate = Date(28, Sep, 2010);
    Settings::instance().evaluationDate() = settlementDate;

    const Size maturityInWeeks = 52;
    const Size maturityInDays  = 7*maturityInWeeks;
    const Size maturityInHours = 24*maturityInDays;
    Date maturityDate = settlementDate + Period(maturityInWeeks, Weeks);

    std::vector<Time> exerciseTimes;
    exerciseTimes.reserve(maturityInHours);

    std::vector<Real> eexPrices, ttfPrices;
    std::vector<Real> eexShape, ttfShape;
    eexShape.reserve(maturityInHours);
    ttfShape.reserve(maturityInHours);
    eexPrices.reserve(maturityInHours);
    ttfPrices.reserve(maturityInHours);

    // read shape from file
    // format is Date, 0h price, 1h price, 2h price, ... 23h price
    // example:4-Oct-2010 36.73 32.09 27.32 23.22 25.71 35.60 47.32 ...
    std::ifstream eex("EEX_2010-2013.txt");
    QL_REQUIRE(eex.is_open(), "can't find price file EEX_2010-2013.txt");
    
    std::ifstream ttf("TTF_2010-2013.txt");
    QL_REQUIRE(ttf.is_open(), "can't find price file TTF_2010-2013.txt");

    Real p;
    for (Size i=0; i < maturityInDays; ++i) {
        std::string str;
        eex >> str;
        ttf >> str;
        
        Date date = settlementDate+Period(i, Days);
        for (Size j=0; j < 24; ++j) {
            Time t = dc.yearFraction(settlementDate, date);
            const Time dt = dc.yearFraction(settlementDate, date+Period(1u, Days))-t;
            t += j*dt/24.;
            exerciseTimes.push_back(t);
            
            eex >> p;
            eexPrices.push_back(p);
            eexShape.push_back(std::log(p)
                -square<Real>()(volatility_x)/(4*alpha)*(1-std::exp(-2*alpha*t))
                -lambda/beta*std::log((eta-std::exp(-beta*t))/(eta-1.0)));
            
            ttf >> p;
            ttfPrices.push_back(p);
            ttfShape.push_back(std::log(p)
                -square<Real>()(volatility_u)/(4*kappa)*(1-std::exp(-2*kappa*t)));
        }
    }
    eex.close();
    ttf.close();
 
    Array x0(2);
    x0[0] = 0.0; x0[1] = 0.0;

    TimeGrid grid(exerciseTimes.begin(), exerciseTimes.end(), 1);
    boost::shared_ptr<ExtendedOrnsteinUhlenbeckProcess> ouProcess(
               new ExtendedOrnsteinUhlenbeckProcess(alpha, volatility_x, x0[0],
                                                    constant<Real, Real>(x0[0])));
    boost::shared_ptr<ExtOUWithJumpsProcess> lnPowerProcess(
                      new ExtOUWithJumpsProcess(ouProcess, x0[1], beta, lambda, eta));
    
    Real u=0.0;
    boost::shared_ptr<ExtendedOrnsteinUhlenbeckProcess> lnGasProcess(
               new ExtendedOrnsteinUhlenbeckProcess(kappa, volatility_u, u,
                                                    constant<Real, Real>(u)));
    
    const Time dt       = 1/(365*24.);
    const Time sqrtDt   = std::sqrt(dt);
    const Real dSqrtRho = std::sqrt(1-rho*rho);

    std::vector<Real> eexSim, ttfSim;
    eexSim.reserve(maturityInHours);
    ttfSim.reserve(maturityInHours);

    PseudoRandom::rng_type rng 
        = PseudoRandom::rng_type(PseudoRandom::urng_type(11234567ul));


    for (Size i=0; i < maturityInHours; ++i) {
        const Time t = i/(365.*24);

        Array z(3);
        z[0] = rng.next().value;
        z[1] = rng.next().value;
        z[2] = rng.next().value;
        const Real z4 = rho*z[0] + dSqrtRho*rng.next().value;

        x0 = lnPowerProcess->evolve(t, x0, dt, z);
        u = lnGasProcess->evolve(t, u, dt, z4);

        eexSim.push_back(std::exp(x0[0]+x0[1]+eexShape[i]));
        ttfSim.push_back(std::exp(u+ttfShape[i]));
    }

    // embedded R script. Remove the following lines if you don't use RInside
    RInside R(argc, argv);

    R["eex"] = eexSim;
    R["ttf"] = ttfSim;

    std::string rScript[] = {

        "if (nchar(installed.packages()[,'Package']['Cairo'])==5) { "
        "  library(\"Cairo\"); ",
        "  if (Cairo.capabilities()['png']) { "
        "    CairoPNG(\"plot.png\") "
        "  } else { "
        "  pdf(\"plot.pdf\") }"
        "} else { pdf(\"plot.pdf\") }", 

        "par(mar=c(4,4,2,2)+0.1, mfcol=c(2,1))",

        "plot(eex, lty=1, type=\"l\",xlab=\"Time (in h)\","
        "ylab=\"EEX Price (in EUR/MWh)\")",

        "plot(ttf, lty=1, type=\"l\",xlab=\"Time (in h)\","
        "ylab=\"TTF Price (in EUR/MWh)\")"
    };


    for (Size i=0; i < sizeof(rScript)/sizeof(std::string); ++i) {
        R.parseEvalQ(rScript[i]);
    }

    return 0;
}
